@@ -13,25 +13,7 @@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -76,7 +58,7 @@ class ReplyBackImExporter implements IImportChanges, IExportChanges {
      * @access public
      * @throws StatusException
      */
-    public function ReplyBackImExporter($session, $store, $folderid) {
+    public function __construct($session, $store, $folderid) {
         $this->session = $session;
         $this->store = $store;
         $this->folderid = $folderid;
@@ -486,6 +468,18 @@ class ReplyBackImExporter implements IImportChanges, IExportChanges {
         try {
             ZLog::Write(LOGLEVEL_DEBUG, sprintf("ReplyBackImExporter->getMessage(): Getting message from MAPIProvider, sourcekey: '%s', parentsourcekey: '%s', entryid: '%s'", bin2hex($sourcekey), bin2hex($parentsourcekey), bin2hex($entryid)));
             $message = $this->mapiprovider->GetMessage($mapimessage, $this->contentparameters);
+
+            // strip or do not send private messages from shared folders to the device
+            if (MAPIUtils::IsMessageSharedAndPrivate($this->folderid, $mapimessage)) {
+                if ($message->SupportsPrivateStripping()) {
+                    ZLog::Write(LOGLEVEL_DEBUG, "ReplyBackImExporter->getMessage(): stripping data of private message from a shared folder");
+                    $message->StripData(Streamer::STRIP_PRIVATE_DATA);
+                }
+                else {
+                    ZLog::Write(LOGLEVEL_DEBUG, "ReplyBackImExporter->getMessage(): ignoring private message from a shared folder");
+                    return SYNC_E_IGNORE;
+                }
+            }
         }
         catch (SyncObjectBrokenException $mbe) {
             if ($announceErrors) {
@@ -515,7 +509,7 @@ class ReplyBackImExporter implements IImportChanges, IExportChanges {
      * @return void
      */
     private function sendNotificationEmail($message, $oldmessage) {
-        // get email address and full name of the user
+        // get email address and full name of the user that performed the operation (auth user in all cases)
         $userinfo = ZPush::GetBackend()->GetUserDetails(Request::GetAuthUser());
 
         // get the name of the folder
