@@ -52,7 +52,7 @@
  * @author     Sean Coates <sean@php.net>
  * @copyright  2003-2006 PEAR <pear-group@php.net>
  * @license    http://www.opensource.org/licenses/bsd-license.php BSD License
- * @version    CVS: $Id: mimeDecode.php 288500 2009-09-21 05:32:32Z alan_k $
+ * @version    CVS: $Id: mimeDecode.php 337165 2015-07-15 09:42:08Z alan_k $
  * @link       http://pear.php.net/package/Mail_mime
  */
 
@@ -61,12 +61,14 @@
  * Z-Push changes
  *
  * removed PEAR dependency by implementing own raiseError()
- * removed deprecated referencing: $obj = &new Mail_mimeDecode($body); in _decode()
  * implemented automated decoding of strings from mail charset
  *
  * Reference implementation used:
- * http://download.pear.php.net/package/Mail_mimeDecode-1.5.1.tgz
+ * http://download.pear.php.net/package/Mail_mimeDecode-1.5.6.tgz
  *
+ * used "old" method of checking if called statically, as this is deprecated between php 5.0.0 and 5.3.0
+ *   (isStatic of decode() around line 215)
+ * Changed constructor name to __construct
  */
 
 /**
@@ -152,12 +154,13 @@ class Mail_mimeDecode
 
     /**
      * Flag to determine whether to decode headers
-     *
-     * @var    boolean
+     * (set to UTF8 to iconv convert headers)
+     * @var    mixed
      * @access private
      */
     var $_decode_headers;
 
+
     /**
      * Flag to determine whether to include attached messages
      * as body in the returned object. Depends on $_include_bodies
@@ -176,7 +179,7 @@ class Mail_mimeDecode
      * @param string The input to decode
      * @access public
      */
-    function Mail_mimeDecode($input, $deprecated_linefeed = '')
+    function __construct($input)
     {
         list($header, $body)   = $this->_splitBodyHeader($input);
 
@@ -187,6 +190,11 @@ class Mail_mimeDecode
         $this->_include_bodies = true;
         $this->_rfc822_bodies  = false;
     }
+    // BC
+    function Mail_mimeDecode($input)
+    {
+        $this->__construct($input);
+    }
 
     /**
      * Begins the decoding process. If called statically
@@ -199,7 +207,9 @@ class Mail_mimeDecode
      *                               object.
      *              decode_bodies  - Whether to decode the bodies
      *                               of the parts. (Transfer encoding)
-     *              decode_headers - Whether to decode headers
+     *              decode_headers - Whether to decode headers,
+     *                             - use "UTF8//IGNORE" to convert charset.
+     *
      *              input          - If called statically, this will be treated
      *                               as the input
      *              charset        - convert all data to this charset
@@ -209,7 +219,7 @@ class Mail_mimeDecode
     function decode($params = null)
     {
         // determine if this method has been called statically
-        $isStatic = !(isset($this) && get_class($this) == __CLASS__);
+        $isStatic = empty($this) || !is_a($this, __CLASS__);
 
         // Have we been called statically?
         // If so, create an object and pass details to that.
@@ -235,6 +245,13 @@ class Mail_mimeDecode
             $this->_charset = isset($params['charset']) ?
                                  strtolower($params['charset']) : 'utf-8';
 
+            if (is_string($this->_decode_headers)) {
+                if (!function_exists('iconv')) {
+                    $this->raiseError('header decode conversion requested, however iconv is missing');
+                }
+                $this->_decode_headers = strtolower($this->_decode_headers);
+            }
+
             $structure = $this->_decode($this->_header, $this->_body);
             if ($structure === false) {
                 $structure = $this->raiseError($this->_error);
@@ -261,6 +278,7 @@ class Mail_mimeDecode
         $headers = $this->_parseHeaders($headers);
 
         foreach ($headers as $value) {
+            $value['value'] =  $this->_decodeHeader($value['value']);
             if (isset($return->headers[strtolower($value['name'])]) AND !is_array($return->headers[strtolower($value['name'])])) {
                 $return->headers[strtolower($value['name'])]   = array($return->headers[strtolower($value['name'])]);
                 $return->headers[strtolower($value['name'])][] = $value['value'];
@@ -273,21 +291,26 @@ class Mail_mimeDecode
             }
         }
 
-        reset($headers);
-        while (list($key, $value) = each($headers)) {
+
+        foreach ($headers as $key => $value) {
             $headers[$key]['name'] = strtolower($headers[$key]['name']);
             switch ($headers[$key]['name']) {
 
                 case 'content-type':
                     $content_type = $this->_parseHeaderValue($headers[$key]['value']);
 
-                    if (preg_match('/([0-9a-z+.-]+)\/([0-9a-z+.-]+)/i', $content_type['value'], $regs)) {
+                    if (preg_match('/([0-9a-z+.-]+)\/([0-9a-z+.-]+)\; name=\"([0-9a-z+.-]+)/i', $headers[$key]['value'], $regs)) {
+                        $return->ctype_primary   = $regs[1];
+                        $return->ctype_secondary = $regs[2];
+                        $return->filename = $regs[3];
+                    }
+                    elseif (preg_match('/([0-9a-z+.-]+)\/([0-9a-z+.-]+)/i', $content_type['value'], $regs)) {
                         $return->ctype_primary   = $regs[1];
                         $return->ctype_secondary = $regs[2];
                     }
 
                     if (isset($content_type['other'])) {
-                        while (list($p_name, $p_value) = each($content_type['other'])) {
+                        foreach($content_type['other'] as $p_name => $p_value) {
                             $return->ctype_parameters[$p_name] = $p_value;
                         }
                     }
@@ -297,7 +320,7 @@ class Mail_mimeDecode
                     $content_disposition = $this->_parseHeaderValue($headers[$key]['value']);
                     $return->disposition   = $content_disposition['value'];
                     if (isset($content_disposition['other'])) {
-                        while (list($p_name, $p_value) = each($content_disposition['other'])) {
+                        foreach($content_disposition['other'] as $p_name => $p_value) {
                             $return->d_parameters[$p_name] = $p_value;
                         }
                     }
@@ -314,15 +337,24 @@ class Mail_mimeDecode
                 case 'text/plain':
                     $encoding = isset($content_transfer_encoding) ? $content_transfer_encoding['value'] : '7bit';
                     $charset = isset($return->ctype_parameters['charset']) ? $return->ctype_parameters['charset'] : $this->_charset;
-                    $this->_include_bodies ? $return->body = ($this->_decode_bodies ? $this->_decodeBody($body, $encoding, $charset) : $body) : null;
+                    $this->_include_bodies ? $return->body = ($this->_decode_bodies ? $this->_decodeBody($body, $encoding, $charset, true) : $body) : null;
                     break;
 
                 case 'text/html':
                     $encoding = isset($content_transfer_encoding) ? $content_transfer_encoding['value'] : '7bit';
                     $charset = isset($return->ctype_parameters['charset']) ? $return->ctype_parameters['charset'] : $this->_charset;
-                    $this->_include_bodies ? $return->body = ($this->_decode_bodies ? $this->_decodeBody($body, $encoding, $charset) : $body) : null;
+                    $this->_include_bodies ? $return->body = ($this->_decode_bodies ? $this->_decodeBody($body, $encoding, $charset, true) : $body) : null;
                     break;
 
+                case 'multipart/signed': // PGP
+                    $parts = $this->_boundarySplit($body, $content_type['other']['boundary'], true);
+                    $return->parts['msg_body'] = $parts[0];
+                    list($part_header, $part_body) = $this->_splitBodyHeader($parts[1]);
+                    $return->parts['sig_hdr'] = $part_header;
+                    $return->parts['sig_body'] = $part_body;
+                    break;
+
+                case 'multipart/encrypted': // #190 encrypted parts will be treated as normal ones
                 case 'multipart/parallel':
                 case 'multipart/appledouble': // Appledouble mail
                 case 'multipart/report': // RFC1892
@@ -330,7 +362,9 @@ class Mail_mimeDecode
                 case 'multipart/digest':
                 case 'multipart/alternative':
                 case 'multipart/related':
+                case 'multipart/relative': //#20431 - android
                 case 'multipart/mixed':
+                case 'application/vnd.wap.multipart.related':
                     if(!isset($content_type['other']['boundary'])){
                         $this->_error = 'No boundary found for ' . $content_type['value'] . ' part';
                         return false;
@@ -349,10 +383,11 @@ class Mail_mimeDecode
                     break;
 
                 case 'message/rfc822':
+                case 'message/delivery-status': // #bug #18693
                     if ($this->_rfc822_bodies) {
                         $encoding = isset($content_transfer_encoding) ? $content_transfer_encoding['value'] : '7bit';
                         $charset = isset($return->ctype_parameters['charset']) ? $return->ctype_parameters['charset'] : $this->_charset;
-                        $return->body = ($this->_decode_bodies ? $this->_decodeBody($body, $encoding, $charset) : $body);
+                        $return->body = ($this->_decode_bodies ? $this->_decodeBody($body, $encoding, $charset, false) : $body);
                     }
 
                     $obj = new Mail_mimeDecode($body);
@@ -360,14 +395,35 @@ class Mail_mimeDecode
                                                           'decode_bodies'  => $this->_decode_bodies,
                                                           'decode_headers' => $this->_decode_headers));
                     unset($obj);
+
+                    // #213, KD 2015-06-29 - Always inline them because there is no "type" to them (they're text)
+                    $return->disposition = 'inline';
                     break;
 
+                // #190, KD 2015-06-09 - Add type for S/MIME Encrypted messages; these must have the filename set explicitly (it won't work otherwise)
+                // and then falls through for the rest on purpose.
+                case 'application/x-pkcs7-mime':
+                case 'application/pkcs7-mime':
+                    if (!isset($content_transfer_encoding['value'])) {
+                        $content_transfer_encoding['value'] = 'base64';
+                    }
+                    // if there is no explicit charset, then don't try to convert to default charset, and make sure that only text mimetypes are converted
+                    $charset = (isset($return->ctype_parameters['charset']) && ((isset($return->ctype_primary) && $return->ctype_primary == 'text') || !isset($return->ctype_primary)) ) ? $return->ctype_parameters['charset'] : '';
+                    $part->body = ($this->_decode_bodies ? $this->_decodeBody($body, $content_transfer_encoding['value'], $charset, false) : $body);
+                    $ctype = explode('/', strtolower($content_type['value']));
+                    $part->ctype_parameters['name'] = 'smime.p7m';
+                    $part->ctype_primary = $ctype[0];
+                    $part->ctype_secondary = $ctype[1];
+                    $part->d_parameters['size'] = strlen($part->body);
+                    $return->parts[] = $part;
+                    // Fall through intentionally
+
                 default:
                     if(!isset($content_transfer_encoding['value']))
                         $content_transfer_encoding['value'] = '7bit';
                     // if there is no explicit charset, then don't try to convert to default charset, and make sure that only text mimetypes are converted
                     $charset = (isset($return->ctype_parameters['charset']) && ((isset($return->ctype_primary) && $return->ctype_primary == 'text') || !isset($return->ctype_primary)) )? $return->ctype_parameters['charset']: '';
-                    $this->_include_bodies ? $return->body = ($this->_decode_bodies ? $this->_decodeBody($body, $content_transfer_encoding['value'], $charset) : $body) : null;
+                    $this->_include_bodies ? $return->body = ($this->_decode_bodies ? $this->_decodeBody($body, $content_transfer_encoding['value'], $charset, false) : $body) : null;
                     break;
             }
 
@@ -437,6 +493,11 @@ class Mail_mimeDecode
         if (preg_match("/^(.*?)\r?\n\r?\n(.*)/s", $input, $match)) {
             return array($match[1], $match[2]);
         }
+        // bug #17325 - empty bodies are allowed. - we just check that at least one line
+        // of headers exist..
+        if (count(explode("\n",$input))) {
+            return array($input, '');
+        }
         $this->_error = 'Could not split header and body';
         return false;
     }
@@ -451,22 +512,39 @@ class Mail_mimeDecode
      */
     function _parseHeaders($input)
     {
-
         if ($input !== '') {
             // Unfold the input
             $input   = preg_replace("/\r?\n/", "\r\n", $input);
+            //#7065 - wrapping.. with encoded stuff.. - probably not needed,
+            // wrapping space should only get removed if the trailing item on previous line is a
+            // encoded character
+            $input   = preg_replace("/=\r\n(\t| )+/", '=', $input);
             $input   = preg_replace("/\r\n(\t| )+/", ' ', $input);
-            $headers = explode("\r\n", trim($input));
 
+            $headers = explode("\r\n", trim($input));
+            $got_start = false;
             foreach ($headers as $value) {
+                if (!$got_start) {
+                    // munge headers for mbox style from
+                    if ($value[0] == '>') {
+                        $value = substring($value, 1); // remove mbox >
+                    }
+                    if (substr($value,0,5) == 'From ') {
+                        $value = 'Return-Path: ' . substr($value, 5);
+                    } else {
+                        $got_start = true;
+                    }
+                }
+
                 $hdr_name = substr($value, 0, $pos = strpos($value, ':'));
                 $hdr_value = substr($value, $pos+1);
-                if($hdr_value[0] == ' ')
+                if($hdr_value[0] == ' ') {
                     $hdr_value = substr($hdr_value, 1);
+                }
 
                 $return[] = array(
                                   'name'  => $hdr_name,
-                                  'value' => $this->_decode_headers ? $this->_decodeHeader($hdr_value) : $hdr_value
+                                  'value' =>  $hdr_value
                                  );
             }
         } else {
@@ -483,49 +561,190 @@ class Mail_mimeDecode
      * robust as it could be. Eg. header comments
      * in the wrong place will probably break it.
      *
+     * Extra things this can handle
+     *   filename*0=......
+     *   filename*1=......
+     *
+     *  This is where lines are broken in, and need merging.
+     *
+     *   filename*0*=ENC'lang'urlencoded data.
+     *   filename*1*=ENC'lang'urlencoded data.
+     *
+     *
+     *
      * @param string Header value to parse
      * @return array Contains parsed result
      * @access private
      */
     function _parseHeaderValue($input)
     {
-
-        if (($pos = strpos($input, ';')) !== false) {
-
-            $return['value'] = trim(substr($input, 0, $pos));
-            $input = trim(substr($input, $pos+1));
-
-            if (strlen($input) > 0) {
-
-                // This splits on a semi-colon, if there's no preceeding backslash
-                // Now works with quoted values; had to glue the \; breaks in PHP
-                // the regex is already bordering on incomprehensible
-                //$splitRegex = '/([^;\'"]*[\'"]([^\'"]*([^\'"]*)*)[\'"][^;\'"]*|([^;]+))(;|$)/';
-                // simplyfied RegEx - Nokia Mail2 sends boundaries containing ' which break the above regex
-                $splitRegex = '/([^;\'"]*[\'"]([^\'"]*)[\'"][^;\'"]*|([^;]+))(;|$)/';
-                preg_match_all($splitRegex, $input, $matches);
-
-                $parameters = array();
-                for ($i=0; $i<count($matches[0]); $i++) {
-                    $param = $matches[0][$i];
-                    while (substr($param, -2) == '\;') {
-                        $param .= $matches[0][++$i];
-                    }
-                    $parameters[] = $param;
-                }
-
-                for ($i = 0; $i < count($parameters); $i++) {
-                    $param_name  = trim(substr($parameters[$i], 0, $pos = strpos($parameters[$i], '=')), "'\";\t\\ ");
-                    $param_value = trim(str_replace('\;', ';', substr($parameters[$i], $pos + 1)), "'\";\t\\ ");
-                    if (!empty($param_value[0]) && $param_value[0] == '"') {
-                        $param_value = substr($param_value, 1, -1);
-                    }
-                    $return['other'][$param_name] = $param_value;
-                    $return['other'][strtolower($param_name)] = $param_value;
-                }
-            }
-        } else {
+         if (($pos = strpos($input, ';')) === false) {
+            $input = $this->_decodeHeader($input);
             $return['value'] = trim($input);
+            return $return;
+        }
+
+
+
+        $value = substr($input, 0, $pos);
+        $value = $this->_decodeHeader($value);
+        $return['value'] = trim($value);
+        $input = trim(substr($input, $pos+1));
+
+        if (!strlen($input) > 0) {
+            return $return;
+        }
+        // at this point input contains xxxx=".....";zzzz="...."
+        // since we are dealing with quoted strings, we need to handle this properly..
+        $i = 0;
+        $l = strlen($input);
+        $key = '';
+        $val = false; // our string - including quotes..
+        $q = false; // in quote..
+        $lq = ''; // last quote..
+
+        while ($i < $l) {
+
+            $c = $input[$i];
+            //var_dump(array('i'=>$i,'c'=>$c,'q'=>$q, 'lq'=>$lq, 'key'=>$key, 'val' =>$val));
+
+            $escaped = false;
+            if ($c == '\\') {
+                $i++;
+                if ($i == $l-1) { // end of string.
+                    break;
+                }
+                $escaped = true;
+                $c = $input[$i];
+            }
+
+
+            // state - in key..
+            if ($val === false) {
+                if (!$escaped && $c == '=') {
+                    $val = '';
+                    $key = trim($key);
+                    $i++;
+                    continue;
+                }
+                if (!$escaped && $c == ';') {
+                    if ($key) { // a key without a value..
+                        $key= trim($key);
+                        $return['other'][$key] = '';
+                    }
+                    $key = '';
+                }
+                $key .= $c;
+                $i++;
+                continue;
+            }
+
+            // state - in value.. (as $val is set..)
+
+            if ($q === false) {
+                // not in quote yet.
+                if ((!strlen($val) || $lq !== false) && $c == ' ' ||  $c == "\t") {
+                    $i++;
+                    continue; // skip leading spaces after '=' or after '"'
+                }
+
+                // do not de-quote 'xxx*= itesm..
+                $key_is_trans = $key[strlen($key)-1] == '*';
+
+                if (!$key_is_trans && !$escaped && ($c == '"' || $c == "'")) {
+                    // start quoted area..
+                    $q = $c;
+                    // in theory should not happen raw text in value part..
+                    // but we will handle it as a merged part of the string..
+                    $val = !strlen(trim($val)) ? '' : trim($val);
+                    $i++;
+                    continue;
+                }
+                // got end....
+                if (!$escaped && $c == ';') {
+
+                    $return['other'][$key] = trim($val);
+                    $val = false;
+                    $key = '';
+                    $lq = false;
+                    $i++;
+                    continue;
+                }
+
+                $val .= $c;
+                $i++;
+                continue;
+            }
+
+            // state - in quote..
+            if (!$escaped && $c == $q) {  // potential exit state..
+
+                // end of quoted string..
+                $lq = $q;
+                $q = false;
+                $i++;
+                continue;
+            }
+
+            // normal char inside of quoted string..
+            $val.= $c;
+            $i++;
+        }
+
+        // do we have anything left..
+        if (strlen(trim($key)) || $val !== false) {
+
+            $val = trim($val);
+
+            $return['other'][$key] = $val;
+        }
+
+
+        $clean_others = array();
+        // merge added values. eg. *1[*]
+        foreach($return['other'] as $key =>$val) {
+            if (preg_match('/\*[0-9]+\**$/', $key)) {
+                $key = preg_replace('/(.*)\*[0-9]+(\**)$/', '\1\2', $key);
+                if (isset($clean_others[$key])) {
+                    $clean_others[$key] .= $val;
+                    continue;
+                }
+
+            }
+            $clean_others[$key] = $val;
+
+        }
+
+        // handle language translation of '*' ending others.
+        foreach( $clean_others as $key =>$val) {
+            if ( $key[strlen($key)-1] != '*') {
+                $clean_others[strtolower($key)] = $val;
+                continue;
+            }
+            unset($clean_others[$key]);
+            $key = substr($key,0,-1);
+            //extended-initial-value := [charset] "'" [language] "'"
+            //              extended-other-values
+            $match = array();
+            $info = preg_match("/^([^']+)'([^']*)'(.*)$/", $val, $match);
+
+            $clean_others[$key] = urldecode($match[3]);
+            $clean_others[strtolower($key)] = $clean_others[$key];
+            $clean_others[strtolower($key).'-charset'] = $match[1];
+            $clean_others[strtolower($key).'-language'] = $match[2];
+
+
+        }
+
+
+        $return['other'] = $clean_others;
+
+        // decode values.
+        foreach($return['other'] as $key =>$val) {
+            $charset = isset($return['other'][$key . '-charset']) ?
+                $return['other'][$key . '-charset']  : false;
+
+            $return['other'][$key] = $this->_decodeHeader($val, $charset);
         }
 
         return $return;
@@ -539,7 +758,7 @@ class Mail_mimeDecode
      * @return array Contains array of resulting mime parts
      * @access private
      */
-    function _boundarySplit($input, $boundary)
+    function _boundarySplit($input, $boundary, $eatline = false)
     {
         $parts = array();
 
@@ -549,13 +768,22 @@ class Mail_mimeDecode
         if ($boundary == $bs_check) {
             $boundary = $bs_possible;
         }
+        // eatline is used by multipart/signed.
+        $tmp = $eatline ?
+            preg_split("/\r?\n--".preg_quote($boundary, '/')."(|--)\n/", $input) :
+            preg_split("/--".preg_quote($boundary, '/')."((?=\s)|--)/", $input);
 
-        $tmp = explode('--' . $boundary, $input);
-
-        for ($i = 1; $i < count($tmp) - 1; $i++) {
-            $parts[] = $tmp[$i];
+        $len = count($tmp) -1;
+        for ($i = 1; $i < $len; $i++) {
+            if (strlen(trim($tmp[$i]))) {
+                $parts[] = $tmp[$i];
+            }
         }
 
+        // add the last part on if it does not end with the 'closing indicator'
+        if (!empty($tmp[$len]) && strlen(trim($tmp[$len])) && $tmp[$len][0] != '-') {
+            $parts[] = $tmp[$len];
+        }
         return $parts;
     }
 
@@ -569,20 +797,22 @@ class Mail_mimeDecode
      * @return string Decoded header value
      * @access private
      */
-    function _decodeHeader($input)
+    function _decodeHeader($input, $default_charset=false)
     {
+        if (!$this->_decode_headers) {
+            return $input;
+        }
         // Remove white space between encoded-words
         $input = preg_replace('/(=\?[^?]+\?(q|b)\?[^?]*\?=)(\s)+=\?/i', '\1=?', $input);
 
         // For each encoded-word...
         while (preg_match('/(=\?([^?]+)\?(q|b)\?([^?]*)\?=)/i', $input, $matches)) {
-
             $encoded  = $matches[1];
-            $charset  = $matches[2];
-            $encoding = $matches[3];
+            $charset  = strtolower($matches[2]);
+            $encoding = strtolower($matches[3]);
             $text     = $matches[4];
 
-            switch (strtolower($encoding)) {
+            switch ($encoding) {
                 case 'b':
                     $text = base64_decode($text);
                     break;
@@ -590,12 +820,20 @@ class Mail_mimeDecode
                 case 'q':
                     $text = str_replace('_', ' ', $text);
                     preg_match_all('/=([a-f0-9]{2})/i', $text, $matches);
-                    foreach($matches[1] as $value)
-                        $text = str_replace('='.$value, chr(hexdec($value)), $text);
+                    foreach ($matches[1] as $value)
+                        $text = str_replace('=' . $value, chr(hexdec($value)), $text);
                     break;
             }
+            if (is_string($this->_decode_headers) && $charset != $this->_decode_headers) {
+                $conv = @iconv($charset, $this->_decode_headers, $text);
+                $text = ($conv === false) ? $text : $conv;
+            }
+            $input = str_replace($encoded, $text, $input);
+        }
 
-            $input = str_replace($encoded, $this->_fromCharset($charset, $text), $input);
+        if ($default_charset  && is_string($this->_decode_headers) && $charset != $this->_decode_headers) {
+            $conv = @iconv($charset, $this->_decode_headers, $input); // TODO: shouldn't this be $default_charset ?
+            $input = ($conv === false) ? $input : $conv;
         }
 
         return $input;
@@ -607,31 +845,28 @@ class Mail_mimeDecode
      *
      * @param  string Input body to decode
      * @param  string Encoding type to use.
+     * @param  string Charset
+     * @param  boolean Must try to autodetect the real charset used
      * @return string Decoded body
      * @access private
      */
-    function _decodeBody($input, $encoding = '7bit', $charset = '')
+    function _decodeBody($input, $encoding = '7bit', $charset = '', $detectCharset = true)
     {
         switch (strtolower($encoding)) {
-            case '7bit':
-                return $this->_fromCharset($charset, $input);;
-                break;
-
-            case '8bit':
-                return $this->_fromCharset($charset, $input);
-                break;
-
             case 'quoted-printable':
-                return $this->_fromCharset($charset, $this->_quotedPrintableDecode($input));
+                $input = $this->_quotedPrintableDecode($input);
                 break;
 
             case 'base64':
-                return $this->_fromCharset($charset, base64_decode($input));
+                $input = base64_decode($input);
                 break;
-
-            default:
-                return $input;
         }
+        if ($detectCharset && strtolower($charset) != $this->_charset) {
+            $conv = @iconv($charset, $this->_charset, $input);
+            $input = ($conv === false) ? $input : $conv;
+        }
+
+        return $input;
     }
 
     /**
@@ -648,7 +883,10 @@ class Mail_mimeDecode
         $input = preg_replace("/=\r?\n/", '', $input);
 
         // Replace encoded characters
-        $input = preg_replace('/=([a-f0-9]{2})/ie', "chr(hexdec('\\1'))", $input);
+
+        $cb = create_function('$matches',  ' return chr(hexdec($matches[0]));');
+
+        $input = preg_replace_callback( '/=([a-f0-9]{2})/i', $cb, $input);
 
         return $input;
     }
@@ -728,6 +966,42 @@ class Mail_mimeDecode
         return $files;
     }
 
+    /**
+     * Get all parts in the message with specified type and concatenate them together, unless the
+     * Content-Disposition is 'attachment', in which case the text is apparently an attachment.
+     *
+     * @param string        $message        mimedecode message(part)
+     * @param string        $message        message subtype
+     * @param string        &$body          body reference
+     * @param boolean       $replace_nr     replace \n\r with \n
+     *
+     * @return void
+     * @access public
+     */
+    static function getBodyRecursive($message, $subtype, &$body, $replace_nr = false) {
+        // TODO: move this function into general utils
+        if (!isset($message->ctype_primary)) return;
+        if (strcasecmp($message->ctype_primary, "text") == 0 && strcasecmp($message->ctype_secondary, $subtype) == 0 && isset($message->body)) {
+            if ($replace_nr) {
+                $body .= str_replace("\n", "\r\n", str_replace("\r", "", $message->body));
+            }
+            else {
+                $body .= $message->body;
+            }
+        }
+
+        if (strcasecmp($message->ctype_primary,"multipart") == 0 && isset($message->parts) && is_array($message->parts)) {
+            foreach($message->parts as $part) {
+                // Check testing/samples/m1009.txt
+                // Content-Type: text/plain; charset=us-ascii; name="hareandtoroise.txt" Content-Transfer-Encoding: 7bit Content-Disposition: inline; filename="hareandtoroise.txt"
+                // We don't want to show that file text (outlook doesn't show it), so if we have content-disposition we don't apply recursivity
+                if (!isset($part->disposition))  {
+                    Mail_mimeDecode::getBodyRecursive($part, $subtype, $body, $replace_nr);
+                }
+            }
+        }
+    }
+
     /**
      * getSendArray() returns the arguments required for Mail::send()
      * used to build the arguments for a mail::send() call
@@ -889,25 +1163,6 @@ class Mail_mimeDecode
         return $return;
     }
 
-    /**
-     * Z-Push helper to decode text
-     *
-     * @param  string  current charset of input
-     * @param  string  input
-     * @return string  XML version of input
-     * @access private
-     */
-    function _fromCharset($charset, $input) {
-        if($charset == '' || (strtolower($charset) == $this->_charset))
-            return $input;
-
-        // all ISO-8859-1 are converted as if they were Windows-1252 - see Mantis #456
-        if (strtolower($charset) == 'iso-8859-1')
-            $charset = 'Windows-1252';
-
-        return @iconv($charset, $this->_charset. "//TRANSLIT", $input);
-    }
-
     /**
      * Z-Push helper for error logging
      * removing PEAR dependency
@@ -920,4 +1175,5 @@ class Mail_mimeDecode
         ZLog::Write(LOGLEVEL_ERROR, "mimeDecode error: ". $message);
         return false;
     }
+
 } // End of class
\ No newline at end of file
