@@ -6,29 +6,11 @@
 *
 * Created   :   12.04.2011
 *
-* Copyright 2007 - 2015 Zarafa Deutschland GmbH
+* Copyright 2007 - 2016 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,7 +23,6 @@
 * Consult LICENSE file for details
 ************************************************/
 
-
 class ZPush {
     const UNAUTHENTICATED = 1;
     const UNPROVISIONED = 2;
@@ -63,6 +44,7 @@ class ZPush {
     const ASV_12 = "12.0";
     const ASV_121 = "12.1";
     const ASV_14 = "14.0";
+    const ASV_141 = "14.1";
 
     /**
      * Command codes for base64 encoded requests (AS >= 12.1)
@@ -97,12 +79,13 @@ class ZPush {
     // Webservice commands
     const COMMAND_WEBSERVICE_DEVICE = -100;
     const COMMAND_WEBSERVICE_USERS = -101;
+    const COMMAND_WEBSERVICE_INFO = -102;
 
     // Latest supported State version
     const STATE_VERSION = IStateMachine::STATEVERSION_02;
 
     static private $autoloadBackendPreference = array(
-                    "BackendZarafa",
+                    "BackendKopano",
                     "BackendCombined",
                     "BackendIMAP",
                     "BackendVCardDir",
@@ -113,7 +96,8 @@ class ZPush {
     static private $supportedASVersions = array(
                     self::ASV_12,
                     self::ASV_121,
-                    self::ASV_14
+                    self::ASV_14,
+                    self::ASV_141
                 );
 
     static private $supportedCommands = array(
@@ -145,6 +129,7 @@ class ZPush {
 
                     self::COMMAND_WEBSERVICE_DEVICE => array(self::REQUESTHANDLER => "Webservice", self::PLAININPUT, self::NOACTIVESYNCCOMMAND, self::WEBSERVICECOMMAND),
                     self::COMMAND_WEBSERVICE_USERS  => array(self::REQUESTHANDLER => "Webservice", self::PLAININPUT, self::NOACTIVESYNCCOMMAND, self::WEBSERVICECOMMAND),
+                    self::COMMAND_WEBSERVICE_INFO   => array(self::REQUESTHANDLER => "Webservice", self::PLAININPUT, self::NOACTIVESYNCCOMMAND, self::WEBSERVICECOMMAND),
             );
 
 
@@ -162,7 +147,7 @@ class ZPush {
                                         self::CLASS_NAME => "SyncContact",
                                         self::CLASS_REQUIRESPROTOCOLVERSION => true,
                                         self::CLASS_DEFAULTTYPE => SYNC_FOLDER_TYPE_CONTACT,
-                                        self::CLASS_OTHERTYPES => array(SYNC_FOLDER_TYPE_USER_CONTACT),
+                                        self::CLASS_OTHERTYPES => array(SYNC_FOLDER_TYPE_USER_CONTACT, SYNC_FOLDER_TYPE_UNKNOWN),
                                    ),
                     "Calendar"  => array(
                                         self::CLASS_NAME => "SyncAppointment",
@@ -191,6 +176,7 @@ class ZPush {
     static private $topCollector;
     static private $backend;
     static private $addSyncFolders;
+    static private $policies;
 
 
     /**
@@ -202,8 +188,9 @@ class ZPush {
      */
     static public function CheckConfig() {
         // check the php version
-        if (version_compare(phpversion(),'5.1.0') < 0)
-            throw new FatalException("The configured PHP version is too old. Please make sure at least PHP 5.1 is used.");
+        if (version_compare(phpversion(),'5.4.0') < 0) {
+            throw new FatalException("The configured PHP version is too old. Please make sure at least PHP 5.4 is used.");
+        }
 
         // some basic checks
         if (!defined('BASE_PATH'))
@@ -220,37 +207,88 @@ class ZPush {
         else
             define('REAL_BASE_PATH', BASE_PATH);
 
-        if (!defined('LOGFILEDIR'))
-            throw new FatalMisconfigurationException("The LOGFILEDIR is not configured. Check if the config.php file is in place.");
+        if (!defined('LOGBACKEND')) {
+            define('LOGBACKEND', 'filelog');
+        }
 
-        if (substr(LOGFILEDIR, -1,1) != "/")
-            throw new FatalMisconfigurationException("The LOGFILEDIR should terminate with a '/'");
+        if (strtolower(LOGBACKEND) == 'syslog') {
+            define('LOGBACKEND_CLASS', 'Syslog');
+            if (!defined('LOG_SYSLOG_FACILITY')) {
+                define('LOG_SYSLOG_FACILITY', LOG_LOCAL0);
+            }
 
-        if (!file_exists(LOGFILEDIR))
-            throw new FatalMisconfigurationException("The configured LOGFILEDIR does not exist or can not be accessed.");
+            if (!defined('LOG_SYSLOG_HOST')) {
+                define('LOG_SYSLOG_HOST', false);
+            }
 
-        if ((!file_exists(LOGFILE) && !touch(LOGFILE)) || !is_writable(LOGFILE))
-            throw new FatalMisconfigurationException("The configured LOGFILE can not be modified.");
+            if (!defined('LOG_SYSLOG_PORT')) {
+                define('LOG_SYSLOG_PORT', 514);
+            }
 
-        if ((!file_exists(LOGERRORFILE) && !touch(LOGERRORFILE)) || !is_writable(LOGERRORFILE))
-            throw new FatalMisconfigurationException("The configured LOGERRORFILE can not be modified.");
+            if (!defined('LOG_SYSLOG_PROGRAM')) {
+                define('LOG_SYSLOG_PROGRAM', 'z-push');
+            }
 
-        // check ownership on the (eventually) just created files
-        Utils::FixFileOwner(LOGFILE);
-        Utils::FixFileOwner(LOGERRORFILE);
+            if (!is_numeric(LOG_SYSLOG_PORT)) {
+                throw new FatalMisconfigurationException("The LOG_SYSLOG_PORT must a be a number.");
+            }
+
+            if (LOG_SYSLOG_HOST && LOG_SYSLOG_PORT <= 0) {
+                throw new FatalMisconfigurationException("LOG_SYSLOG_HOST is defined but the LOG_SYSLOG_PORT does not seem to be valid.");
+            }
+        }
+        elseif (strtolower(LOGBACKEND) == 'filelog') {
+            define('LOGBACKEND_CLASS', 'FileLog');
+            if (!defined('LOGFILEDIR'))
+                throw new FatalMisconfigurationException("The LOGFILEDIR is not configured. Check if the config.php file is in place.");
+
+            if (substr(LOGFILEDIR, -1,1) != "/")
+                throw new FatalMisconfigurationException("The LOGFILEDIR should terminate with a '/'");
+
+            if (!file_exists(LOGFILEDIR))
+                throw new FatalMisconfigurationException("The configured LOGFILEDIR does not exist or can not be accessed.");
+
+            if ((!file_exists(LOGFILE) && !touch(LOGFILE)) || !is_writable(LOGFILE))
+                throw new FatalMisconfigurationException("The configured LOGFILE can not be modified.");
+
+            if ((!file_exists(LOGERRORFILE) && !touch(LOGERRORFILE)) || !is_writable(LOGERRORFILE))
+                throw new FatalMisconfigurationException("The configured LOGERRORFILE can not be modified.");
+
+            // check ownership on the (eventually) just created files
+            Utils::FixFileOwner(LOGFILE);
+            Utils::FixFileOwner(LOGERRORFILE);
+        }
+        else {
+            define('LOGBACKEND_CLASS', LOGBACKEND);
+        }
 
         // set time zone
-        // code contributed by Robert Scheck (rsc) - more information: https://developer.berlios.de/mantis/view.php?id=479
-        if(function_exists("date_default_timezone_set")) {
-            if(defined('TIMEZONE') ? constant('TIMEZONE') : false) {
-                if (! @date_default_timezone_set(TIMEZONE))
-                    throw new FatalMisconfigurationException(sprintf("The configured TIMEZONE '%s' is not valid. Please check supported timezones at http://www.php.net/manual/en/timezones.php", constant('TIMEZONE')));
+        // code contributed by Robert Scheck (rsc)
+        if(defined('TIMEZONE') ? constant('TIMEZONE') : false) {
+            if (! @date_default_timezone_set(TIMEZONE))
+                throw new FatalMisconfigurationException(sprintf("The configured TIMEZONE '%s' is not valid. Please check supported timezones at http://www.php.net/manual/en/timezones.php", constant('TIMEZONE')));
+        }
+        else if(!ini_get('date.timezone')) {
+            date_default_timezone_set('Europe/Amsterdam');
+        }
+
+        // check if Provisioning is enabled and the default policies are available
+        if (PROVISIONING) {
+            if (file_exists(REAL_BASE_PATH . PROVISIONING_POLICYFILE)) {
+                $policyfile = REAL_BASE_PATH . PROVISIONING_POLICYFILE;
             }
-            else if(!ini_get('date.timezone')) {
-                date_default_timezone_set('Europe/Amsterdam');
+            else {
+                $policyfile = PROVISIONING_POLICYFILE;
+            }
+            ZPush::$policies = parse_ini_file($policyfile, true);
+            if (!isset(ZPush::$policies['default'])) {
+                throw new FatalMisconfigurationException(sprintf("Your policies' configuration file doesn't contain the required [default] section. Please check the '%s' file.", $policyfile));
             }
         }
 
+        if (defined('USE_X_FORWARDED_FOR_HEADER')) {
+            ZLog::Write(LOGLEVEL_INFO, "The configuration parameter 'USE_X_FORWARDED_FOR_HEADER' was deprecated in favor of 'USE_CUSTOM_REMOTE_IP_HEADER'. Please update your configuration.");
+        }
         return true;
     }
 
@@ -267,12 +305,6 @@ class ZPush {
         if (!is_array($specialLogUsers))
             throw new FatalMisconfigurationException("The WBXML log users is not an array.");
 
-        if (!defined('SINK_FORCERECHECK')) {
-            define('SINK_FORCERECHECK', 300);
-        }
-        else if (SINK_FORCERECHECK !== false && (!is_int(SINK_FORCERECHECK) || SINK_FORCERECHECK < 1))
-            throw new FatalMisconfigurationException("The SINK_FORCERECHECK value must be 'false' or a number higher than 0.");
-
         if (!defined('SYNC_CONTACTS_MAXPICTURESIZE')) {
             define('SYNC_CONTACTS_MAXPICTURESIZE', 49152);
         }
@@ -283,13 +315,66 @@ class ZPush {
             define('USE_PARTIAL_FOLDERSYNC', false);
         }
 
+        if (!defined('PING_LOWER_BOUND_LIFETIME')) {
+            define('PING_LOWER_BOUND_LIFETIME', false);
+        }
+        elseif(PING_LOWER_BOUND_LIFETIME !== false && (!is_int(PING_LOWER_BOUND_LIFETIME) || PING_LOWER_BOUND_LIFETIME < 1 || PING_LOWER_BOUND_LIFETIME > 3540)){
+            throw new FatalMisconfigurationException("The PING_LOWER_BOUND_LIFETIME value must be 'false' or a number between 1 and 3540 inclusively.");
+        }
+        if (!defined('PING_HIGHER_BOUND_LIFETIME')) {
+            define('PING_HIGHER_BOUND_LIFETIME', false);
+        }
+        elseif(PING_HIGHER_BOUND_LIFETIME !== false && (!is_int(PING_HIGHER_BOUND_LIFETIME) || PING_HIGHER_BOUND_LIFETIME < 1 || PING_HIGHER_BOUND_LIFETIME > 3540)){
+            throw new FatalMisconfigurationException("The PING_HIGHER_BOUND_LIFETIME value must be 'false' or a number between 1 and 3540 inclusively.");
+        }
+        if(PING_HIGHER_BOUND_LIFETIME !== false && PING_LOWER_BOUND_LIFETIME !== false && PING_HIGHER_BOUND_LIFETIME < PING_LOWER_BOUND_LIFETIME){
+            throw new FatalMisconfigurationException("The PING_HIGHER_BOUND_LIFETIME value must be greater or equal to PING_LOWER_BOUND_LIFETIME.");
+        }
+
+        if (!defined('RETRY_AFTER_DELAY')) {
+            define('RETRY_AFTER_DELAY', 300);
+        }
+        elseif (RETRY_AFTER_DELAY !== false && (!is_int(RETRY_AFTER_DELAY) || RETRY_AFTER_DELAY < 1))  {
+            throw new FatalMisconfigurationException("The RETRY_AFTER_DELAY value must be 'false' or a number greater than 0.");
+        }
+
+        // Check KOE flags
+        if (!defined('KOE_CAPABILITY_GAB')) {
+            define('KOE_CAPABILITY_GAB', false);
+        }
+        if (!defined('KOE_CAPABILITY_RECEIVEFLAGS')) {
+            define('KOE_CAPABILITY_RECEIVEFLAGS', false);
+        }
+        if (!defined('KOE_CAPABILITY_SENDFLAGS')) {
+            define('KOE_CAPABILITY_SENDFLAGS', false);
+        }
+        if (!defined('KOE_CAPABILITY_OOF')) {
+            define('KOE_CAPABILITY_OOF', false);
+        }
+        if (!defined('KOE_CAPABILITY_OOFTIMES')) {
+            define('KOE_CAPABILITY_OOFTIMES', false);
+        }
+        if (!defined('KOE_CAPABILITY_NOTES')) {
+            define('KOE_CAPABILITY_NOTES', false);
+        }
+        if (!defined('KOE_CAPABILITY_SHAREDFOLDER')) {
+            define('KOE_CAPABILITY_SHAREDFOLDER', false);
+        }
+        if (!defined('KOE_GAB_FOLDERID')) {
+            define('KOE_GAB_FOLDERID', '');
+        }
+        if (!defined('KOE_GAB_STORE')) {
+            define('KOE_GAB_STORE', '');
+        }
+        if (!defined('KOE_GAB_NAME')) {
+            define('KOE_GAB_NAME', false);
+        }
+
         // the check on additional folders will not throw hard errors, as this is probably changed on live systems
         if (isset($additionalFolders) && !is_array($additionalFolders))
             ZLog::Write(LOGLEVEL_ERROR, "ZPush::CheckConfig() : The additional folders synchronization not available as array.");
         else {
-            self::$addSyncFolders = array();
-
-            // process configured data
+            // check configured data
             foreach ($additionalFolders as $af) {
 
                 if (!is_array($af) || !isset($af['store']) || !isset($af['folderid']) || !isset($af['name']) || !isset($af['type'])) {
@@ -302,22 +387,12 @@ class ZPush {
                     continue;
                 }
 
-                if (!in_array($af['type'], array(SYNC_FOLDER_TYPE_USER_CONTACT, SYNC_FOLDER_TYPE_USER_APPOINTMENT, SYNC_FOLDER_TYPE_USER_TASK, SYNC_FOLDER_TYPE_USER_MAIL))) {
+                if (!in_array($af['type'], array(SYNC_FOLDER_TYPE_USER_NOTE, SYNC_FOLDER_TYPE_USER_CONTACT, SYNC_FOLDER_TYPE_USER_APPOINTMENT, SYNC_FOLDER_TYPE_USER_TASK, SYNC_FOLDER_TYPE_USER_MAIL))) {
                     ZLog::Write(LOGLEVEL_ERROR, sprintf("ZPush::CheckConfig() : the type of the additional synchronization folder '%s is not permitted.", $af['name']));
                     continue;
                 }
-
-                $folder = new SyncFolder();
-                $folder->serverid = $af['folderid'];
-                $folder->parentid = 0;                  // only top folders are supported
-                $folder->displayname = $af['name'];
-                $folder->type = $af['type'];
-                // save store as custom property which is not streamed directly to the device
-                $folder->NoBackendFolder = true;
-                $folder->Store = $af['store'];
-                self::$addSyncFolders[$folder->serverid] = $folder;
+                // the data will be inizialized when used via self::getAddFolders()
             }
-
         }
 
         ZLog::Write(LOGLEVEL_DEBUG, sprintf("Used timezone '%s'", date_default_timezone_get()));
@@ -350,13 +425,17 @@ class ZPush {
             }
             else {
                 // Initialize the default StateMachine
-                include_once('lib/default/filestatemachine.php');
-                ZPush::$stateMachine = new FileStateMachine();
+                if (defined('STATE_MACHINE') && STATE_MACHINE == 'SQL') {
+                    ZPush::$stateMachine = new SqlStateMachine();
+                }
+                else {
+                    ZPush::$stateMachine = new FileStateMachine();
+                }
             }
 
             if (ZPush::$stateMachine->GetStateVersion() !== ZPush::GetLatestStateVersion()) {
                 if (class_exists("TopCollector")) self::GetTopCollector()->AnnounceInformation("Run migration script!", true);
-                throw new HTTPReturnCodeException(sprintf("The state version available to the %s is not the latest version - please run the state upgrade script. See release notes for more information.", get_class(ZPush::$stateMachine), 503));
+                throw new ServiceUnavailableException(sprintf("The state version available to the %s is not the latest version - please run the state upgrade script. See release notes for more information.", get_class(ZPush::$stateMachine)));
             }
         }
         return ZPush::$stateMachine;
@@ -387,6 +466,21 @@ class ZPush {
         return ZPush::$deviceManager;
     }
 
+    /**
+     * Load another device in the DeviceManager and return it.
+     *
+     * @param ASDevice $asDevice
+     * @param boolean $initialize - default: true
+     * @return DeviceManager
+     */
+    static public function GetDeviceManagerWithDevice($asDevice, $initialize = true) {
+        $dm = ZPush::GetDeviceManager($initialize);
+        if ($dm) {
+            $dm->SetDevice($asDevice);
+        }
+        return $dm;
+    }
+
     /**
      * Returns the Top data collector object
      *
@@ -429,8 +523,7 @@ class ZPush {
             return false;
 
         ZLog::Write(LOGLEVEL_DEBUG, sprintf("Including backend file: '%s'", $toLoad));
-        include_once($toLoad);
-        return true;
+        return include_once($toLoad);
     }
 
     /**
@@ -477,25 +570,25 @@ class ZPush {
     static public function GetBackend() {
         // if the backend is not yet loaded, load backend drivers and instantiate it
         if (!isset(ZPush::$backend)) {
+
             // Initialize our backend
             $ourBackend = @constant('BACKEND_PROVIDER');
 
             // if no backend provider is defined, try to include automatically
             if ($ourBackend == false || $ourBackend == "") {
-                $loaded = false;
                 foreach (self::$autoloadBackendPreference as $autoloadBackend) {
                     ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPush::GetBackend(): trying autoload backend '%s'", $autoloadBackend));
-                    $loaded = class_exists($autoloadBackend) || self::IncludeBackend($autoloadBackend);
-                    if ($loaded) {
+                    if (class_exists($autoloadBackend)) {
                         $ourBackend = $autoloadBackend;
                         break;
                     }
                 }
-                if (!$ourBackend || !$loaded)
-                    throw new FatalMisconfigurationException("No Backend provider can not be loaded. Check your installation and configuration!");
+                if (!$ourBackend)
+                    throw new FatalMisconfigurationException("No Backend provider can be found. Check your installation and/or configuration!");
+            }
+            elseif (!class_exists($ourBackend)) {
+                \ZPush::IncludeBackend($ourBackend);
             }
-            elseif (!class_exists($ourBackend))
-                self::IncludeBackend($ourBackend);
 
             if (class_exists($ourBackend))
                 ZPush::$backend = new $ourBackend();
@@ -508,27 +601,48 @@ class ZPush {
     /**
      * Returns additional folder objects which should be synchronized to the device
      *
+     * @param boolean $backendIdsAsKeys     if true the keys are backendids else folderids, default: true
+     *
      * @access public
      * @return array
      */
-    static public function GetAdditionalSyncFolders() {
-        // TODO if there are any user based folders which should be synchronized, they have to be returned here as well!!
-        return self::$addSyncFolders;
+    static public function GetAdditionalSyncFolders($backendIdsAsKeys = true) {
+        // get user based folders which should be synchronized
+        $userFolder = self::GetDeviceManager()->GetAdditionalUserSyncFolders();
+        $addfolders = self::getAddSyncFolders() + $userFolder;
+        // if requested, we rewrite the backendids to folderids here
+        if ($backendIdsAsKeys === false && !empty($addfolders)) {
+            ZLog::Write(LOGLEVEL_DEBUG, "ZPush::GetAdditionalSyncFolders(): Requested AS folderids as keys for additional folders array, converting");
+            $faddfolders = array();
+            foreach ($addfolders as $backendId => $addFolder) {
+                $fid = self::GetDeviceManager()->GetFolderIdForBackendId($backendId);
+                $faddfolders[$fid] = $addFolder;
+            }
+            $addfolders = $faddfolders;
+        }
+
+        return $addfolders;
     }
 
     /**
      * Returns additional folder objects which should be synchronized to the device
      *
-     * @param string        $folderid
+     * @param string        $backendid
      * @param boolean       $noDebug        (opt) by default, debug message is shown
      *
      * @access public
      * @return string
      */
-    static public function GetAdditionalSyncFolderStore($folderid, $noDebug = false) {
-        $val = (isset(self::$addSyncFolders[$folderid]->Store))? self::$addSyncFolders[$folderid]->Store : false;
+    static public function GetAdditionalSyncFolderStore($backendid, $noDebug = false) {
+        if(isset(self::getAddSyncFolders()[$backendid]->Store)) {
+            $val = self::getAddSyncFolders()[$backendid]->Store;
+        }
+        else {
+            $val = self::GetDeviceManager()->GetAdditionalUserSyncFolderStore($backendid);
+        }
+
         if (!$noDebug)
-            ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPush::GetAdditionalSyncFolderStore('%s'): '%s'", $folderid, Utils::PrintAsString($val)));
+            ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPush::GetAdditionalSyncFolderStore('%s'): '%s'", $backendid, Utils::PrintAsString($val)));
         return $val;
     }
 
@@ -552,6 +666,52 @@ class ZPush {
             return new $class();
     }
 
+    /**
+     * Initializes the SyncObjects for additional folders on demand.
+     * Uses DeviceManager->BuildSyncFolderObject() to do patching required for ZP-907.
+     *
+     * @access private
+     * @return array
+     */
+    static private function getAddSyncFolders() {
+        global $additionalFolders;
+        if (!isset(self::$addSyncFolders)) {
+            self::$addSyncFolders = array();
+
+            if (isset($additionalFolders) && !is_array($additionalFolders)) {
+                ZLog::Write(LOGLEVEL_ERROR, "ZPush::getAddSyncFolders() : The additional folders synchronization not available as array.");
+            }
+            else {
+                foreach ($additionalFolders as $af) {
+                    if (!is_array($af) || !isset($af['store']) || !isset($af['folderid']) || !isset($af['name']) || !isset($af['type'])) {
+                        ZLog::Write(LOGLEVEL_ERROR, "ZPush::getAddSyncFolders() : the additional folder synchronization is not configured correctly. Missing parameters. Entry will be ignored.");
+                        continue;
+                    }
+
+                    if ($af['store'] == "" || $af['folderid'] == "" || $af['name'] == "" || $af['type'] == "") {
+                        ZLog::Write(LOGLEVEL_WARN, "ZPush::getAddSyncFolders() : the additional folder synchronization is not configured correctly. Empty parameters. Entry will be ignored.");
+                        continue;
+                    }
+
+                    if (!in_array($af['type'], array(SYNC_FOLDER_TYPE_USER_NOTE, SYNC_FOLDER_TYPE_USER_CONTACT, SYNC_FOLDER_TYPE_USER_APPOINTMENT, SYNC_FOLDER_TYPE_USER_TASK, SYNC_FOLDER_TYPE_USER_MAIL))) {
+                        ZLog::Write(LOGLEVEL_ERROR, sprintf("ZPush::getAddSyncFolders() : the type of the additional synchronization folder '%s is not permitted.", $af['name']));
+                        continue;
+                    }
+
+                    // don't fail hard if no flags are set, but we at least warn about it
+                    if (!isset($af['flags'])) {
+                        ZLog::Write(LOGLEVEL_WARN, sprintf("ZPush::getAddSyncFolders() : the additional folder '%s' is not configured completely. Missing 'flags' parameter, defaulting to DeviceManager::FLD_FLAGS_NONE.", $af['name']));
+                        $af['flags'] = DeviceManager::FLD_FLAGS_NONE;
+                    }
+
+                    $folder = self::GetDeviceManager()->BuildSyncFolderObject($af['store'], $af['folderid'], '0', $af['name'], $af['type'], $af['flags'], DeviceManager::FLD_ORIGIN_CONFIG);
+                    self::$addSyncFolders[$folder->BackendId] = $folder;
+                }
+            }
+        }
+        return self::$addSyncFolders;
+    }
+
     /**
      * Returns the default foldertype for a folder class
      *
@@ -820,5 +980,13 @@ END;
         return $defcapa;
     }
 
+    /**
+     * Returns the available provisioning policies.
+     *
+     * @return array
+     */
+    static public function GetPolicies() {
+        // TODO another policy providers might be available, e.g. for sqlstatemachine
+        return ZPush::$policies;
+    }
 }
-?>
\ No newline at end of file
