@@ -6,29 +6,11 @@
 *
 * Created   :   16.02.2012
 *
-* Copyright 2007 - 2013 Zarafa Deutschland GmbH
+* Copyright 2007 - 2016 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -78,19 +60,23 @@ class FolderChange extends RequestProcessor {
 
         // ServerID
         $serverid = false;
+        $backendid = false;
         if(self::$decoder->getElementStartTag(SYNC_FOLDERHIERARCHY_SERVERENTRYID)) {
             $serverid = self::$decoder->getElementContent();
+            $backendid = self::$deviceManager->GetBackendIdForFolderId($serverid);
             if(!self::$decoder->getElementEndTag())
                 return false;
         }
 
         // Parent
         $parentid = false;
+        $parentBackendId = false;
 
         // when creating or updating more information is necessary
         if (!$delete) {
             if(self::$decoder->getElementStartTag(SYNC_FOLDERHIERARCHY_PARENTID)) {
                 $parentid = self::$decoder->getElementContent();
+                $parentBackendId = self::$deviceManager->GetBackendIdForFolderId($parentid);
                 if(!self::$decoder->getElementEndTag())
                     return false;
             }
@@ -121,24 +107,31 @@ class FolderChange extends RequestProcessor {
             $syncstate = self::$deviceManager->GetStateManager()->GetSyncState($synckey);
             $newsynckey = self::$deviceManager->GetStateManager()->GetNewSyncKey($synckey);
 
+            // there are no SyncParameters for the hierarchy, but we use it to save the latest synckeys
+            $spa = self::$deviceManager->GetStateManager()->GetSynchedFolderState(false);
+
             // Over the ChangesWrapper the HierarchyCache is notified about all changes
             $changesMem = self::$deviceManager->GetHierarchyChangesWrapper();
 
             // the hierarchyCache should now fully be initialized - check for changes in the additional folders
-            $changesMem->Config(ZPush::GetAdditionalSyncFolders());
+            $changesMem->Config(ZPush::GetAdditionalSyncFolders(false));
+
+            // reset to default store in backend
+            self::$backend->Setup(false);
 
             // there are unprocessed changes in the hierarchy, trigger resync
             if ($changesMem->GetChangeCount() > 0)
                 throw new StatusException("HandleFolderChange() can not proceed as there are unprocessed hierarchy changes", SYNC_FSSTATUS_SERVERERROR);
 
-            // any additional folders can not be modified!
-            if ($serverid !== false && ZPush::GetAdditionalSyncFolderStore($serverid))
+            // any additional folders can not be modified - with exception if they are of type SYNC_FOLDER_TYPE_UNKNOWN (ZP-907)
+            if (self::$deviceManager->GetFolderTypeFromCacheById($serverid) != SYNC_FOLDER_TYPE_UNKNOWN && $serverid !== false && ZPush::GetAdditionalSyncFolderStore($backendid))
                 throw new StatusException("HandleFolderChange() can not change additional folders which are configured", SYNC_FSSTATUS_SYSTEMFOLDER);
 
             // switch user store if this this happens inside an additional folder
             // if this is an additional folder the backend has to be setup correctly
-            if (!self::$backend->Setup(ZPush::GetAdditionalSyncFolderStore((($parentid != false)?$parentid:$serverid))))
-                throw new StatusException(sprintf("HandleFolderChange() could not Setup() the backend for folder id '%s'", (($parentid != false)?$parentid:$serverid)), SYNC_FSSTATUS_SERVERERROR);
+            // backend should also not be switched when type is SYNC_FOLDER_TYPE_UNKNOWN (ZP-1220)
+            if (self::$deviceManager->GetFolderTypeFromCacheById($serverid) != SYNC_FOLDER_TYPE_UNKNOWN && !self::$backend->Setup(ZPush::GetAdditionalSyncFolderStore((($parentBackendId != false) ? $parentBackendId : $backendid))))
+                throw new StatusException(sprintf("HandleFolderChange() could not Setup() the backend for folder id '%s'", (($parentBackendId != false) ? $parentBackendId : $backendid)), SYNC_FSSTATUS_SERVERERROR);
         }
         catch (StateNotFoundException $snfex) {
             $status = SYNC_FSSTATUS_SYNCKEYERROR;
@@ -160,20 +153,27 @@ class FolderChange extends RequestProcessor {
                 // the messages from the PIM will be forwarded to the real importer
                 $changesMem->SetDestinationImporter($importer);
 
+                // Create SyncFolder object
+                $folder = new SyncFolder();
+                $folder->serverid = $serverid;
+                $folder->parentid = $parentBackendId;
+                if (isset($displayname)) {
+                    $folder->displayname = $displayname;
+                }
+                if (isset($type)) {
+                    $folder->type = $type;
+                }
+                // add the backendId to the SyncFolder object
+                $folder->BackendId = $backendid;
+
                 // process incoming change
                 if (!$delete) {
-                    // Send change
-                    $folder = new SyncFolder();
-                    $folder->serverid = $serverid;
-                    $folder->parentid = $parentid;
-                    $folder->displayname = $displayname;
-                    $folder->type = $type;
-
-                    $serverid = $changesMem->ImportFolderChange($folder);
+                    // when creating, $folder->serverid is false, and the returned id is already mapped by the backend
+                    $folder = $changesMem->ImportFolderChange($folder);
                 }
                 else {
                     // delete folder
-                    $changesMem->ImportFolderDeletion($serverid, 0);
+                    $changesMem->ImportFolderDeletion($folder);
                 }
             }
             catch (StatusException $stex) {
@@ -196,7 +196,7 @@ class FolderChange extends RequestProcessor {
                     self::$encoder->endTag();
 
                     self::$encoder->startTag(SYNC_FOLDERHIERARCHY_SERVERENTRYID);
-                    self::$encoder->content($serverid);
+                    self::$encoder->content($folder->serverid);
                     self::$encoder->endTag();
                 }
             }
@@ -241,6 +241,11 @@ class FolderChange extends RequestProcessor {
         if (isset($importer)) {
             self::$deviceManager->GetStateManager()->SetSyncState($newsynckey, $importer->GetState());
 
+            // update SPA & save it
+            $spa->SetSyncKey($newsynckey);
+            $spa->SetFolderId(false);
+            self::$deviceManager->GetStateManager()->SetSynchedFolderState($spa);
+
             // invalidate all pingable flags
             SyncCollections::InvalidatePingableFlags();
         }
@@ -248,4 +253,3 @@ class FolderChange extends RequestProcessor {
         return true;
     }
 }
-?>
\ No newline at end of file
