@@ -6,29 +6,11 @@
 *
 * Created   :   16.02.2012
 *
-* Copyright 2007 - 2013 Zarafa Deutschland GmbH
+* Copyright 2007 - 2016 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -52,9 +34,6 @@ class FolderSync extends RequestProcessor {
      * @return boolean
      */
     public function Handle ($commandCode) {
-        // Maps serverid -> clientid for items that are received from the PIM
-        $map = array();
-
         // Parse input
         if(!self::$decoder->getElementStartTag(SYNC_FOLDERHIERARCHY_FOLDERSYNC))
             return false;
@@ -81,6 +60,9 @@ class FolderSync extends RequestProcessor {
 
             // We will be saving the sync state under 'newsynckey'
             $newsynckey = self::$deviceManager->GetStateManager()->GetNewSyncKey($synckey);
+
+            // there are no SyncParameters for the hierarchy, but we use it to save the latest synckeys
+            $spa = self::$deviceManager->GetStateManager()->GetSynchedFolderState(false);
         }
         catch (StateNotFoundException $snfex) {
                 $status = SYNC_FSSTATUS_SYNCKEYERROR;
@@ -95,7 +77,10 @@ class FolderSync extends RequestProcessor {
         $changesMem = self::$deviceManager->GetHierarchyChangesWrapper();
 
         // the hierarchyCache should now fully be initialized - check for changes in the additional folders
-        $changesMem->Config(ZPush::GetAdditionalSyncFolders());
+        $changesMem->Config(ZPush::GetAdditionalSyncFolders(false), ChangesMemoryWrapper::SYNCHRONIZING);
+
+         // reset to default store in backend
+        self::$backend->Setup(false);
 
         // process incoming changes
         if(self::$decoder->getElementStartTag(SYNC_FOLDERHIERARCHY_CHANGES)) {
@@ -113,11 +98,15 @@ class FolderSync extends RequestProcessor {
                 return false;
 
             $importer = false;
-            while(1) {
+            WBXMLDecoder::ResetInWhile("folderSyncIncomingChange");
+            while(WBXMLDecoder::InWhile("folderSyncIncomingChange")) {
                 $folder = new SyncFolder();
                 if(!$folder->Decode(self::$decoder))
                     break;
 
+                // add the backendId to the SyncFolder object
+                $folder->BackendId = self::$deviceManager->GetBackendIdForFolderId($folder->serverid);
+
                 try {
                     if ($status == SYNC_FSSTATUS_SUCCESS && !$importer) {
                         // Configure the backends importer with last state
@@ -137,10 +126,6 @@ class FolderSync extends RequestProcessor {
                                 $serverid = $changesMem->ImportFolderDeletion($folder);
                                 break;
                         }
-
-                        // TODO what does $map??
-                        if($serverid)
-                            $map[$serverid] = $folder->clientid;
                     }
                     else {
                         ZLog::Write(LOGLEVEL_WARN, sprintf("Request->HandleFolderSync(): ignoring incoming folderchange for folder '%s' as status indicates problem.", $folder->displayname));
@@ -188,9 +173,7 @@ class FolderSync extends RequestProcessor {
                     $exporter->InitializeExporter($changesMem);
 
                     // Stream all changes to the ImportExportChangesMem
-                    $maxExporttime = Request::GetExpectedConnectionTimeout();
                     $totalChanges = $exporter->GetChangeCount();
-                    $started = time();
                     $exported = 0;
                     $partial = false;
                     while(is_array($exporter->Synchronize())) {
@@ -201,8 +184,8 @@ class FolderSync extends RequestProcessor {
                         }
 
                         // if partial sync is allowed, stop if this takes too long
-                        if (USE_PARTIAL_FOLDERSYNC && (time() - $started) > $maxExporttime) {
-                            ZLog::Write(LOGLEVEL_WARN, sprintf("Request->HandleFolderSync(): Exporting folders is too slow. In %d seconds only %d from %d changes were processed.",(time() - $started), $exported, $totalChanges));
+                        if (USE_PARTIAL_FOLDERSYNC && Request::IsRequestTimeoutReached()) {
+                            ZLog::Write(LOGLEVEL_WARN, sprintf("Request->HandleFolderSync(): Exporting folders is too slow. In %d seconds only %d from %d changes were processed.",(time() - $_SERVER["REQUEST_TIME"]), $exported, $totalChanges));
                             self::$topCollector->AnnounceInformation(sprintf("Partial export of %d out of %d folders", $exported, $totalChanges), true);
                             self::$deviceManager->SetFolderSyncComplete(false);
                             $partial = true;
@@ -215,7 +198,7 @@ class FolderSync extends RequestProcessor {
                         // say that we are done with partial synching
                         self::$deviceManager->SetFolderSyncComplete(true);
                         // reset the loop data to prevent any loop detection to kick in now
-                        self::$deviceManager->ClearLoopDetectionData(Request::GetAuthUser(), Request::GetDeviceId());
+                        self::$deviceManager->ClearLoopDetectionData(Request::GetAuthUserString(), Request::GetDeviceID());
                         ZLog::Write(LOGLEVEL_INFO, "Request->HandleFolderSync(): Chunked exporting of folders completed successfully");
                     }
 
@@ -255,9 +238,27 @@ class FolderSync extends RequestProcessor {
                 self::$encoder->endTag();
                 self::$topCollector->AnnounceInformation(sprintf("Outgoing %d folders",$changeCount), true);
 
+                if ($changeCount == 0) {
+                    self::$deviceManager->CheckFolderData();
+                }
                 // everything fine, save the sync state for the next time
-                if ($synckey == $newsynckey)
+                if ($synckey == $newsynckey) {
                     self::$deviceManager->GetStateManager()->SetSyncState($newsynckey, $newsyncstate);
+
+                    // update SPA & save it
+                    $spa->SetSyncKey($newsynckey);
+                    $spa->SetFolderId(false);
+
+                    // invalidate all pingable flags
+                    SyncCollections::InvalidatePingableFlags();
+                }
+                // save the SyncParameters if it changed or the reference policy key is not set or different
+                if ($spa->IsDataChanged() || !$spa->HasReferencePolicyKey() || self::$deviceManager->ProvisioningRequired($spa->GetReferencePolicyKey(), true, false)) {
+                    // saves the SPA (while updating the reference policy key)
+                    $spa->SetLastSynctime(time());
+                    self::$deviceManager->GetStateManager()->SetSynchedFolderState($spa);
+                }
+
             }
         }
         self::$encoder->endTag();
@@ -265,4 +266,3 @@ class FolderSync extends RequestProcessor {
         return true;
     }
 }
-?>
\ No newline at end of file
