@@ -6,29 +6,11 @@
 *
 * Created   :   16.02.2012
 *
-* Copyright 2007 - 2013 Zarafa Deutschland GmbH
+* Copyright 2007 - 2016 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -59,7 +41,8 @@ class ItemOperations extends RequestProcessor {
 
         $itemoperations = array();
         //ItemOperations can either be Fetch, EmptyFolderContents or Move
-        while (1) {
+        WBXMLDecoder::ResetInWhile("itemOperationsActions");
+        while(WBXMLDecoder::InWhile("itemOperationsActions")) {
             //TODO check if multiple item operations are possible in one request
             $el = self::$decoder->getElement();
 
@@ -91,7 +74,8 @@ class ItemOperations extends RequestProcessor {
             }
 
             // process operation
-            while(1) {
+            WBXMLDecoder::ResetInWhile("itemOperationsOperation");
+            while(WBXMLDecoder::InWhile("itemOperationsOperation")) {
                 if ($fetch) {
                     if(self::$decoder->getElementStartTag(SYNC_ITEMOPERATIONS_STORE)) {
                         $operation['store'] = self::$decoder->getElementContent();
@@ -134,7 +118,8 @@ class ItemOperations extends RequestProcessor {
 
                         // Save all OPTIONS into a ContentParameters object
                         $operation["cpo"] = new ContentParameters();
-                        while(1) {
+                        WBXMLDecoder::ResetInWhile("itemOperationsOptions");
+                        while(WBXMLDecoder::InWhile("itemOperationsOptions")) {
                             while (self::$decoder->getElementStartTag(SYNC_AIRSYNCBASE_BODYPREFERENCE)) {
                                 if(self::$decoder->getElementStartTag(SYNC_AIRSYNCBASE_TYPE)) {
                                     $bptype = self::$decoder->getElementContent();
@@ -166,6 +151,37 @@ class ItemOperations extends RequestProcessor {
                                     return false;//SYNC_AIRSYNCBASE_BODYPREFERENCE
                             }
 
+                            if (self::$decoder->getElementStartTag(SYNC_AIRSYNCBASE_BODYPARTPREFERENCE)) {
+                                if (self::$decoder->getElementStartTag(SYNC_AIRSYNCBASE_TYPE)) {
+                                    $bpptype = self::$decoder->getElementContent();
+                                    $operation["cpo"]->BodyPartPreference($bpptype);
+                                    if (!self::$decoder->getElementEndTag()) {
+                                        return false;
+                                    }
+                                }
+
+                                if (self::$decoder->getElementStartTag(SYNC_AIRSYNCBASE_TRUNCATIONSIZE)) {
+                                    $operation["cpo"]->BodyPartPreference($bpptype)->SetTruncationSize(self::$decoder->getElementContent());
+                                    if(!self::$decoder->getElementEndTag())
+                                        return false;
+                                }
+
+                                if (self::$decoder->getElementStartTag(SYNC_AIRSYNCBASE_ALLORNONE)) {
+                                    $operation["cpo"]->BodyPartPreference($bpptype)->SetAllOrNone(self::$decoder->getElementContent());
+                                    if(!self::$decoder->getElementEndTag())
+                                        return false;
+                                }
+
+                                if (self::$decoder->getElementStartTag(SYNC_AIRSYNCBASE_PREVIEW)) {
+                                    $operation["cpo"]->BodyPartPreference($bpptype)->SetPreview(self::$decoder->getElementContent());
+                                    if(!self::$decoder->getElementEndTag())
+                                        return false;
+                                }
+
+                                if (!self::$decoder->getElementEndTag())
+                                    return false;
+                            }
+
                             if(self::$decoder->getElementStartTag(SYNC_MIMESUPPORT)) {
                                 $operation["cpo"]->SetMimeSupport(self::$decoder->getElementContent());
                                 if(!self::$decoder->getElementEndTag())
@@ -180,7 +196,8 @@ class ItemOperations extends RequestProcessor {
 
                             if(self::$decoder->getElementStartTag(SYNC_ITEMOPERATIONS_SCHEMA)) {
                                 // read schema tags
-                                while (1) {
+                                WBXMLDecoder::ResetInWhile("itemOperationsSchema");
+                                while(WBXMLDecoder::InWhile("itemOperationsSchema")) {
                                     // TODO save elements
                                     $el = self::$decoder->getElement();
                                     $e = self::$decoder->peek();
@@ -191,6 +208,12 @@ class ItemOperations extends RequestProcessor {
                                 }
                             }
 
+                            if(self::$decoder->getElementStartTag(SYNC_RIGHTSMANAGEMENT_SUPPORT)) {
+                                $operation["cpo"]->SetRmSupport(self::$decoder->getElementContent());
+                                if(!self::$decoder->getElementEndTag())
+                                    return false;
+                            }
+
                             //break if it reached the endtag
                             $e = self::$decoder->peek();
                             if($e[EN_TYPE] == EN_TYPE_ENDTAG) {
@@ -199,6 +222,16 @@ class ItemOperations extends RequestProcessor {
                             }
                         }
                     }
+
+                    if (self::$decoder->getElementStartTag(SYNC_RIGHTSMANAGEMENT_REMOVERIGHTSMGNTPROTECTION)) {
+                        $operation["cpo"]->SetRemoveRmProtection(true);
+                        if (($rrmp = self::$decoder->getElementContent()) !== false) {
+                            $operation["cpo"]->SetRemoveRmProtection($rrmp);
+                            if(!self::$decoder->getElementEndTag()) {
+                                return false;
+                            }
+                        }
+                    }
                 } // end if fetch
 
                 if ($efc) {
@@ -211,7 +244,7 @@ class ItemOperations extends RequestProcessor {
                         if(self::$decoder->getElementStartTag(SYNC_ITEMOPERATIONS_DELETESUBFOLDERS)) {
                             $operation['deletesubfolders'] = true;
                             if (($dsf = self::$decoder->getElementContent()) !== false) {
-                                $operation['deletesubfolders'] = (boolean)$dsf;
+                                $operation['deletesubfolders'] = (bool)$dsf;
                                 if(!self::$decoder->getElementEndTag())
                                     return false;
                             }
@@ -230,6 +263,11 @@ class ItemOperations extends RequestProcessor {
                 }
             } // end while operation
 
+            // rewrite folderid into backendfolderid to be used on backend operations below
+            if (isset($operation['folderid'])) {
+                $operation['backendfolderid'] = self::$deviceManager->GetBackendIdForFolderId($operation['folderid']);
+            }
+
             $itemoperations[] = $operation;
             //break if it reached the endtag
             $e = self::$decoder->peek();
@@ -279,12 +317,12 @@ class ItemOperations extends RequestProcessor {
                     try {
                         if (isset($operation['folderid']) && isset($operation['serverid'])) {
                             self::$topCollector->AnnounceInformation("Fetching data from backend with item and folder id");
-                            $data = self::$backend->Fetch($operation['folderid'], $operation['serverid'], $operation["cpo"]);
+                            $data = self::$backend->Fetch($operation['backendfolderid'], $operation['serverid'], $operation["cpo"]);
                         }
                         else if (isset($operation['longid'])) {
                             self::$topCollector->AnnounceInformation("Fetching data from backend with long id");
                             $tmp = explode(":", $operation['longid']);
-                            $data = self::$backend->Fetch($tmp[0], $tmp[1], $operation["cpo"]);
+                            $data = self::$backend->Fetch(self::$deviceManager->GetBackendIdForFolderId($tmp[0]), $tmp[1], $operation["cpo"]);
                         }
                     }
                     catch (StatusException $stex) {
@@ -345,7 +383,7 @@ class ItemOperations extends RequestProcessor {
                     self::$topCollector->AnnounceInformation("Emptying folder");
 
                     // send request to backend
-                    self::$backend->EmptyFolder($operation['folderid'], $operation['deletesubfolders']);
+                    self::$backend->EmptyFolder($operation['backendfolderid'], $operation['deletesubfolders']);
                 }
                 catch (StatusException $stex) {
                    $status = $stex->getCode();
@@ -384,4 +422,3 @@ class ItemOperations extends RequestProcessor {
         return true;
     }
 }
-?>
\ No newline at end of file
