@@ -229,11 +229,14 @@ class Sync extends RequestProcessor {
 
                     // Get changes can be an empty tag as well as have value
                     // code block partly contributed by dw2412
-                    if(self::$decoder->getElementStartTag(SYNC_GETCHANGES)) {
+                    if($starttag = self::$decoder->getElementStartTag(SYNC_GETCHANGES)) {
                         $sc->AddParameter($spa, "getchanges", true);
                         if (($gc = self::$decoder->getElementContent()) !== false) {
                             $sc->AddParameter($spa, "getchanges", $gc);
-                            if(!self::$decoder->getElementEndTag()) {
+                        }
+                        // read the endtag if SYNC_GETCHANGES wasn't an empty tag
+                        if ($starttag[EN_FLAGS] & EN_FLAGS_CONTENT) {
+                            if (!self::$decoder->getElementEndTag()) {
                                 return false;
                             }
                         }
@@ -369,6 +372,43 @@ class Sync extends RequestProcessor {
                                     return false;
                             }
 
+                            if (self::$decoder->getElementStartTag(SYNC_AIRSYNCBASE_BODYPARTPREFERENCE)) {
+                                if (self::$decoder->getElementStartTag(SYNC_AIRSYNCBASE_TYPE)) {
+                                    $bpptype = self::$decoder->getElementContent();
+                                    $spa->BodyPartPreference($bpptype);
+                                    if (!self::$decoder->getElementEndTag()) {
+                                        return false;
+                                    }
+                                }
+
+                                if (self::$decoder->getElementStartTag(SYNC_AIRSYNCBASE_TRUNCATIONSIZE)) {
+                                    $spa->BodyPartPreference($bpptype)->SetTruncationSize(self::$decoder->getElementContent());
+                                    if(!self::$decoder->getElementEndTag())
+                                        return false;
+                                }
+
+                                if (self::$decoder->getElementStartTag(SYNC_AIRSYNCBASE_ALLORNONE)) {
+                                    $spa->BodyPartPreference($bpptype)->SetAllOrNone(self::$decoder->getElementContent());
+                                    if(!self::$decoder->getElementEndTag())
+                                        return false;
+                                }
+
+                                if (self::$decoder->getElementStartTag(SYNC_AIRSYNCBASE_PREVIEW)) {
+                                    $spa->BodyPartPreference($bpptype)->SetPreview(self::$decoder->getElementContent());
+                                    if(!self::$decoder->getElementEndTag())
+                                        return false;
+                                }
+
+                                if (!self::$decoder->getElementEndTag())
+                                    return false;
+                            }
+
+                            if (self::$decoder->getElementStartTag(SYNC_RIGHTSMANAGEMENT_SUPPORT)) {
+                                $spa->SetRmSupport(self::$decoder->getElementContent());
+                                if (!self::$decoder->getElementEndTag())
+                                    return false;
+                            }
+
                             $e = self::$decoder->peek();
                             if($e[EN_TYPE] == EN_TYPE_ENDTAG) {
                                 self::$decoder->getElementEndTag();
@@ -378,10 +418,11 @@ class Sync extends RequestProcessor {
                     }
 
                     // limit items to be synchronized to the mobiles if configured
-                    if (defined('SYNC_FILTERTIME_MAX') && SYNC_FILTERTIME_MAX > SYNC_FILTERTYPE_ALL &&
-                        (!$spa->HasFilterType() || $spa->GetFilterType() == SYNC_FILTERTYPE_ALL || $spa->GetFilterType() > SYNC_FILTERTIME_MAX)) {
-                            ZLog::Write(LOGLEVEL_DEBUG, sprintf("SYNC_FILTERTIME_MAX defined. Filter set to value: %s", SYNC_FILTERTIME_MAX));
-                            $spa->SetFilterType(SYNC_FILTERTIME_MAX);
+                    $maxAllowed = self::$deviceManager->GetFilterType($spa->GetFolderId());
+                    if ($maxAllowed > SYNC_FILTERTYPE_ALL &&
+                        (!$spa->HasFilterType() || $spa->GetFilterType() == SYNC_FILTERTYPE_ALL || $spa->GetFilterType() > $maxAllowed)) {
+                            ZLog::Write(LOGLEVEL_DEBUG, sprintf("HandleSync(): FilterType applied globally or specifically, using value: %s", $maxAllowed));
+                            $spa->SetFilterType($maxAllowed);
                     }
 
                     // unset filtertype for KOE GAB folder
@@ -391,7 +432,7 @@ class Sync extends RequestProcessor {
                     }
 
                     if ($currentFilterType != $spa->GetFilterType()) {
-                        ZLog::Write(LOGLEVEL_DEBUG, sprintf("HandleSync(): filter type has changed (old: '%s', new: '%s'), removing folderstat to force Exporter setup", $currentFilterType, $spa->GetFilterType()));
+                        ZLog::Write(LOGLEVEL_DEBUG, sprintf("HandleSync(): FilterType has changed (old: '%s', new: '%s'), removing folderstat to force Exporter setup", $currentFilterType, $spa->GetFilterType()));
                         $spa->DelFolderStat();
                     }
 
@@ -631,7 +672,11 @@ class Sync extends RequestProcessor {
             foreach($sc as $folderid => $spa) {
                 // manually set getchanges parameter for this collection if it is synchronized
                 if ($spa->HasSyncKey()) {
-                    $sc->AddParameter($spa, "getchanges", true);
+                    $actiondata = $sc->GetParameter($spa, "actiondata");
+                    // request changes if no other actions are executed
+                    if (empty($actiondata["modifyids"]) && empty($actiondata["clientids"]) && empty($actiondata["removeids"])) {
+                        $sc->AddParameter($spa, "getchanges", true);
+                    }
 
                     // announce WindowSize to DeviceManager
                     self::$deviceManager->SetWindowSize($folderid, $spa->GetWindowSize());
@@ -649,8 +694,8 @@ class Sync extends RequestProcessor {
             }
         }
 
-        // HEARTBEAT & Empty sync
-        if ($status == SYNC_STATUS_SUCCESS && (isset($hbinterval) || $emptysync == true)) {
+        // HEARTBEAT
+        if ($status == SYNC_STATUS_SUCCESS && isset($hbinterval)) {
             $interval = (defined('PING_INTERVAL') && PING_INTERVAL > 0) ? PING_INTERVAL : 30;
 
             if (isset($hbinterval))
@@ -784,6 +829,12 @@ class Sync extends RequestProcessor {
                     $setupExporter = false;
                 }
 
+                // if max memory allocation is reached, stop processing other collections
+                if (Request::IsRequestMemoryLimitReached()) {
+                    ZLog::Write(LOGLEVEL_DEBUG, sprintf("Sync(): no exporter setup for '%s' as max memory allocatation reached, omitting output for collection.", $spa->GetFolderId()));
+                    $setupExporter = false;
+                }
+
                 // ZP-907: never send changes of UNKNOWN folders to an Outlook client
                 if (Request::IsOutlook() && self::$deviceManager->GetFolderTypeFromCacheById($spa->GetFolderId()) == SYNC_FOLDER_TYPE_UNKNOWN && $spa->HasSyncKey()) {
                     ZLog::Write(LOGLEVEL_DEBUG, sprintf("Sync(): no exporter setup for '%s' as type is UNKNOWN.", $spa->GetFolderId()));
@@ -883,7 +934,7 @@ class Sync extends RequestProcessor {
 
             // Fir AS 14.0+ omit output for folder, if there were no incoming or outgoing changes and no Fetch
             if (Request::GetProtocolVersion() >= 14.0 && ! $spa->HasNewSyncKey() && $changecount == 0 && empty($actiondata["fetchids"]) && $status == SYNC_STATUS_SUCCESS &&
-                    ($newFolderStat === false || ! $spa->IsExporterRunRequired($newFolderStat))) {
+                    ! $spa->HasConfirmationChanged() && ($newFolderStat === false || ! $spa->IsExporterRunRequired($newFolderStat))) {
                 ZLog::Write(LOGLEVEL_DEBUG, sprintf("HandleSync: No changes found for %s folder id '%s'. Omitting output.", $spa->GetContentClass(), $spa->GetFolderId()));
                 continue;
             }
@@ -909,6 +960,14 @@ class Sync extends RequestProcessor {
         if ($this->startFolderTagSent)
             self::$encoder->endTag();
 
+        // Check if there was any response - in case of an empty sync request, we shouldn't send an empty answer (ZP-1241)
+        if (!$this->startTagsSent && $emptysync === true) {
+            $this->sendStartTags();
+            self::$encoder->startTag(SYNC_STATUS);
+            self::$encoder->content(SYNC_STATUS_SYNCREQUESTINCOMPLETE);
+            self::$encoder->endTag();
+        }
+
         //SYNC_SYNCHRONIZE - only if the starttag was sent
         if ($this->startTagsSent)
             self::$encoder->endTag();
@@ -1164,7 +1223,7 @@ class Sync extends RequestProcessor {
                     }
                 }
 
-                if($n >= $windowSize || Request::IsRequestTimeoutReached()) {
+                if($n >= $windowSize || Request::IsRequestTimeoutReached() || Request::IsRequestMemoryLimitReached()) {
                     ZLog::Write(LOGLEVEL_DEBUG, sprintf("HandleSync(): Exported maxItems of messages: %d / %d", $n, $changecount));
                     break;
                 }
@@ -1179,9 +1238,9 @@ class Sync extends RequestProcessor {
             self::$encoder->endTag();
 
             // log the request timeout
-            if (Request::IsRequestTimeoutReached()) {
-                ZLog::Write(LOGLEVEL_DEBUG, "HandleSync(): Stopping export as maximum request timeout is almost reached!");
-                // Send a <MoreAvailable/> tag if we reached the request timeout, there are more changes and a moreavailable was not already send
+            if (Request::IsRequestTimeoutReached() || Request::IsRequestMemoryLimitReached()) {
+                ZLog::Write(LOGLEVEL_DEBUG, "HandleSync(): Stopping export as limits of request timeout or available memory are almost reached!");
+                // Send a <MoreAvailable/> tag if we reached the request timeout or max memory, there are more changes and a moreavailable was not already send
                 if (!$moreAvailableSent && ($n > $windowSize)) {
                     self::$encoder->startTag(SYNC_MOREAVAILABLE, false, true);
                     $spa->DelFolderStat();
@@ -1204,12 +1263,14 @@ class Sync extends RequestProcessor {
                 self::$deviceManager->SetFolderSyncStatus($spa->GetFolderId(), DeviceManager::FLD_SYNC_COMPLETED);
 
                 // we should update the folderstat, but we recheck to see if it changed. If so, it's not updated to force another sync
-                $newFolderStatAfterExport = self::$backend->GetFolderStat(ZPush::GetAdditionalSyncFolderStore($spa->GetBackendFolderId()), $spa->GetBackendFolderId());
-                if ($newFolderStat === $newFolderStatAfterExport) {
-                    $this->setFolderStat($spa, $newFolderStat);
-                }
-                else {
-                    ZLog::Write(LOGLEVEL_DEBUG, "Sync() Folderstat differs after export, force another exporter run.");
+                if (self::$backend->HasFolderStats()) {
+                    $newFolderStatAfterExport = self::$backend->GetFolderStat(ZPush::GetAdditionalSyncFolderStore($spa->GetBackendFolderId()), $spa->GetBackendFolderId());
+                    if ($newFolderStat === $newFolderStatAfterExport) {
+                        $this->setFolderStat($spa, $newFolderStat);
+                    }
+                    else {
+                        ZLog::Write(LOGLEVEL_DEBUG, "Sync() Folderstat differs after export, force another exporter run.");
+                    }
                 }
             }
             else
