@@ -61,6 +61,9 @@ class SyncAppointment extends SyncObject {
     public $responsetype;
     public $responserequested;
 
+    // AS 14.1 props
+    public $onlineMeetingConfLink;
+    public $onlineMeetingExternalLink;
 
     function __construct() {
         $mapping = array(
@@ -211,6 +214,13 @@ class SyncAppointment extends SyncObject {
                                                                                     self::STREAMER_RONOTIFY => true);
         }
 
+        if(Request::GetProtocolVersion() >= 14.1) {
+            $mapping[SYNC_POOMCAL_ONLINEMEETINGCONFLINK]                = array (   self::STREAMER_VAR      => "onlineMeetingConfLink",
+                                                                                    self::STREAMER_RONOTIFY => true);
+            $mapping[SYNC_POOMCAL_ONLINEMEETINGEXTERNALLINK]            = array (   self::STREAMER_VAR      => "onlineMeetingExternalLink",
+                                                                                    self::STREAMER_RONOTIFY => true);
+        }
+
         parent::__construct($mapping);
 
         // Indicates that this SyncObject supports the private flag and stripping of private data.
